#pragma once
#pragma once
#ifndef GEOMETRY_SHAPES_H
#define GEOMETRY_SHAPES_H

#include "sgg/graphics.h"
#include <cmath>
#define M_PI 3.14159265358979323846
/**
 * Το namespace GeometryShapes περιέχει συναρτήσεις για τη σχεδίαση
 * βασικών δισδιάστατων γεωμετρικών σχημάτων.
 */
namespace GeometryShapes {

    /**
     * Σχεδιάζει ένα τετράγωνο.
     * @param x Η x συντεταγμένη του κέντρου.
     * @param y Η y συντεταγμένη του κέντρου.
     * @param side Το μήκος της πλευράς.
     * @param brush Το αντικείμενο Brush για καθορισμό του χρώματος.
     */
    void drawSquare(float x, float y, float side, const graphics::Brush& brush) {
        graphics::drawRect(x, y, side, side, brush);
    }

    /**
     * Σχεδιάζει ένα παραλληλόγραμμο.
     * @param x Η x συντεταγμένη του κέντρου.
     * @param y Η y συντεταγμένη του κέντρου.
     * @param width Το πλάτος του παραλληλογράμμου.
     * @param height Το ύψος του παραλληλογράμμου.
     * @param brush Το αντικείμενο Brush για καθορισμό του χρώματος.
     */
    void drawRectangle(float x, float y, float width, float height, const graphics::Brush& brush) {
        graphics::drawRect(x, y, width, height, brush);
    }

    /**
     * Σχεδιάζει ένα ρόμβο.
     * @param x Η x συντεταγμένη του κέντρου.
     * @param y Η y συντεταγμένη του κέντρου.
     * @param diagonal1 Η πρώτη διαγώνιος του ρόμβου.
     * @param diagonal2 Η δεύτερη διαγώνιος του ρόμβου.
     * @param brush Το αντικείμενο Brush για καθορισμό του χρώματος.
     */
    void drawRhombus(float x, float y, float diagonal1, float diagonal2, const graphics::Brush& brush) {
        float halfD1 = diagonal1 / 2;
        float halfD2 = diagonal2 / 2;

        graphics::drawLine(x, y - halfD1, x + halfD2, y, brush); // επάνω δεξιά
        graphics::drawLine(x + halfD2, y, x, y + halfD1, brush); // κάτω δεξιά
        graphics::drawLine(x, y + halfD1, x - halfD2, y, brush); // κάτω αριστερά
        graphics::drawLine(x - halfD2, y, x, y - halfD1, brush); // επάνω αριστερά
    }

    /**
     * Σχεδιάζει ένα ορθογώνιο παραλληλόγραμμο.
     * @param x Η x συντεταγμένη του κέντρου.
     * @param y Η y συντεταγμένη του κέντρου.
     * @param width Το πλάτος του ορθογώνιου παραλληλογράμμου.
     * @param height Το ύψος του ορθογώνιου παραλληλογράμμου.
     * @param brush Το αντικείμενο Brush για καθορισμό του χρώματος.
     */
    void drawParallelogram(float x, float y, float width, float height, const graphics::Brush& brush) {
        float offsetX = width / 4;

        graphics::drawLine(x - offsetX, y - height / 2, x + width - offsetX, y - height / 2, brush); // πάνω γραμμή
        graphics::drawLine(x + width - offsetX, y - height / 2, x + offsetX, y + height / 2, brush); // δεξιά γραμμή
        graphics::drawLine(x + offsetX, y + height / 2, x - width + offsetX, y + height / 2, brush); // κάτω γραμμή
        graphics::drawLine(x - width + offsetX, y + height / 2, x - offsetX, y - height / 2, brush); // αριστερή γραμμή
    }

    /**
     * Σχεδιάζει ένα τραπέζιο.
     * @param x Η x συντεταγμένη του κέντρου.
     * @param y Η y συντεταγμένη του κέντρου.
     * @param topBase Το μήκος της πάνω βάσης.
     * @param bottomBase Το μήκος της κάτω βάσης.
     * @param height Το ύψος του τραπεζίου.
     * @param brush Το αντικείμενο Brush για καθορισμό του χρώματος.
     */
    void drawTrapezoid(float x, float y, float topBase, float bottomBase, float height, const graphics::Brush& brush) {
        float halfHeight = height / 2;
        float halfTopBase = topBase / 2;
        float halfBottomBase = bottomBase / 2;

        graphics::drawLine(x - halfTopBase, y - halfHeight, x + halfTopBase, y - halfHeight, brush); // επάνω γραμμή
        graphics::drawLine(x + halfTopBase, y - halfHeight, x + halfBottomBase, y + halfHeight, brush); // δεξιά γραμμή
        graphics::drawLine(x + halfBottomBase, y + halfHeight, x - halfBottomBase, y + halfHeight, brush); // κάτω γραμμή
        graphics::drawLine(x - halfBottomBase, y + halfHeight, x - halfTopBase, y - halfHeight, brush); // αριστερή γραμμή
    }

    /**
     * Σχεδιάζει ένα τρίγωνο.
     * @param x1, y1 Συντεταγμένες πρώτης κορυφής.
     * @param x2, y2 Συντεταγμένες δεύτερης κορυφής.
     * @param x3, y3 Συντεταγμένες τρίτης κορυφής.
     * @param brush Το αντικείμενο Brush για καθορισμό του χρώματος.
     */
    void drawTriangle(float x1, float y1, float x2, float y2, float x3, float y3, const graphics::Brush& brush) {
        graphics::drawLine(x1, y1, x2, y2, brush);
        graphics::drawLine(x2, y2, x3, y3, brush);
        graphics::drawLine(x3, y3, x1, y1, brush);
    }

    /**
     * Σχεδιάζει ένα ορθογώνιο τρίγωνο.
     * @param x1, y1 Συντεταγμένες της ορθής γωνίας.
     * @param width Το μήκος της βάσης.
     * @param height Το ύψος.
     * @param brush Το αντικείμενο Brush για καθορισμό του χρώματος.
     */
    void drawRightTriangle(float x1, float y1, float width, float height, const graphics::Brush& brush) {
        graphics::drawLine(x1, y1, x1 + width, y1, brush);
        graphics::drawLine(x1 + width, y1, x1, y1 + height, brush);
        graphics::drawLine(x1, y1 + height, x1, y1, brush);
    }

    /**
     * Σχεδιάζει ένα ισοσκελές τρίγωνο.
     * @param x, y Το κέντρο της βάσης του τριγώνου.
     * @param base Το μήκος της βάσης.
     * @param height Το ύψος.
     * @param brush Το αντικείμενο Brush για καθορισμό του χρώματος.
     */
    void drawIsoscelesTriangle(float x, float y, float base, float height, const graphics::Brush& brush) {
        graphics::drawLine(x - base / 2, y, x + base / 2, y, brush);
        graphics::drawLine(x - base / 2, y, x, y - height, brush);
        graphics::drawLine(x + base / 2, y, x, y - height, brush);
    }

    /**
     * Σχεδιάζει ένα ισόπλευρο τρίγωνο.
     * @param x, y Το κέντρο του τριγώνου.
     * @param side Το μήκος της πλευράς.
     * @param brush Το αντικείμενο Brush για καθορισμό του χρώματος.
     */
    void drawEquilateralTriangle(float x, float y, float side, const graphics::Brush& brush) {
        float height = (sqrt(3) / 2) * side;
        float halfSide = side / 2;

        graphics::drawLine(x - halfSide, y + height / 2, x + halfSide, y + height / 2, brush);
        graphics::drawLine(x - halfSide, y + height / 2, x, y - height / 2, brush);
        graphics::drawLine(x + halfSide, y + height / 2, x, y - height / 2, brush);
    }

    /**
     * Σχεδιάζει ένα κανονικό πολύγωνο.
     * @param x Η x συντεταγμένη του κέντρου.
     * @param y Η y συντεταγμένη του κέντρου.
     * @param sides Ο αριθμός των πλευρών.
     * @param radius Η απόσταση από το κέντρο έως τις κορυφές.
     * @param brush Το αντικείμενο Brush για καθορισμό του χρώματος.
     */
    void drawRegularPolygon(float x, float y, int sides, float radius, const graphics::Brush& brush) {
        if (sides < 3) return; // Πρέπει να έχει τουλάχιστον 3 πλευρές
        float angleStep = 2 * M_PI / sides;

        for (int i = 0; i < sides; ++i) {
            float x1 = x + radius * cos(i * angleStep);
            float y1 = y + radius * sin(i * angleStep);
            float x2 = x + radius * cos((i + 1) * angleStep);
            float y2 = y + radius * sin((i + 1) * angleStep);
            graphics::drawLine(x1, y1, x2, y2, brush);
        }
    }

} // namespace GeometryShapes

#endif // GEOMETRY_SHAPES_H
