#pragma once
#ifndef THREE_D_SHAPES_H
#define THREE_D_SHAPES_H

#include "sgg/graphics.h"
#include <cmath>
#define M_PI 3.14159265358979323846

namespace ThreeDShapes {

    /**
     * Σχεδιάζει έναν κύβο σε προοπτική.
     * @param x Η x-συντεταγμένη του κέντρου του κύβου.
     * @param y Η y-συντεταγμένη του κέντρου του κύβου.
     * @param side Το μήκος της πλευράς του κύβου.
     * @param brush Το αντικείμενο Brush που καθορίζει το χρώμα και το στυλ σχεδίασης.
     */
    void drawCube(float x, float y, float side, const graphics::Brush& brush) {
        float offset = side * 0.4f;
        graphics::drawRect(x, y, side, side, brush);
        graphics::drawRect(x + offset, y - offset, side, side, brush);

        graphics::drawLine(x - side / 2, y - side / 2, x + offset - side / 2, y - offset - side / 2, brush);
        //graphics::drawLine(x + side / 2, y - side / 2, x + offset + side / 2, y - offset - side / 2, brush);
        graphics::drawLine(x - side / 2, y + side / 2, x + offset - side / 2, y - offset + side / 2, brush);
        graphics::drawLine(x + side / 2, y + side / 2, x + offset + side / 2, y - offset + side / 2, brush);
    }

    /**
     * Σχεδιάζει ένα ορθογώνιο παραλληλεπίπεδο (Cuboid) σε προοπτική.
     * @param x Η x-συντεταγμένη του κέντρου του cuboid.
     * @param y Η y-συντεταγμένη του κέντρου του cuboid.
     * @param width Το πλάτος του cuboid.
     * @param height Το ύψος του cuboid.
     * @param depth Το βάθος του cuboid.
     * @param brush Το αντικείμενο Brush για το στυλ σχεδίασης.
     */
    void drawCuboid(float x, float y, float width, float height, float depth, const graphics::Brush& brush) {
        float offsetX = depth * 0.4f;
        float offsetY = depth * 0.4f;

        graphics::drawRect(x, y, width, height, brush);
        graphics::drawRect(x + offsetX, y - offsetY, width, height, brush);

        graphics::drawLine(x - width / 2, y - height / 2, x + offsetX - width / 2, y - offsetY - height / 2, brush);
        //graphics::drawLine(x + width / 2, y - height / 2, x + offsetX + width / 2, y - offsetY - height / 2, brush);
        graphics::drawLine(x - width / 2, y + height / 2, x + offsetX - width / 2, y - offsetY + height / 2, brush);
        graphics::drawLine(x + width / 2, y + height / 2, x + offsetX + width / 2, y - offsetY + height / 2, brush);
    }

    /**
     * Σχεδιάζει έναν κώνο σε προοπτική.
     * @param x Η x-συντεταγμένη της βάσης του κώνου.
     * @param y Η y-συντεταγμένη της βάσης του κώνου.
     * @param radius Η ακτίνα της βάσης του κώνου.
     * @param height Το ύψος του κώνου.
     * @param brush Το αντικείμενο Brush για το στυλ σχεδίασης.
     */
    void drawCone(float x, float y, float radius, float height, const graphics::Brush& brush) {
        float base_offset = radius * 0.6f;
        // Σχεδίαση ελλειπτικής βάσης για προοπτική
        graphics::setScale(1.0f, 0.5f);
        graphics::drawDisk(x, y + height / 2, base_offset, brush);
        graphics::resetPose();

        // Σχεδίαση γραμμών από κορυφή προς βάση
        graphics::drawLine(x, y - height / 2, x - base_offset, y + height / 2, brush);
        graphics::drawLine(x, y - height / 2, x + base_offset, y + height / 2, brush);
    }

    /**
     * Σχεδιάζει έναν κύλινδρο σε προοπτική.
     * @param x Η x-συντεταγμένη του κέντρου του κυλίνδρου.
     * @param y Η y-συντεταγμένη του κέντρου του κυλίνδρου.
     * @param radius Η ακτίνα του κυλίνδρου.
     * @param height Το ύψος του κυλίνδρου.
     * @param brush Το αντικείμενο Brush για το στυλ σχεδίασης.
     */
    void drawCylinder(float x, float y, float radius, float height, const graphics::Brush& brush) {
        float base_offset = radius * 0.6f;

        // Σχεδίαση ελλειπτικής κορυφής και βάσης για προοπτική
        graphics::setScale(1.0f, 0.5f);
        graphics::drawDisk(x, y - height / 2, base_offset, brush); // Κορυφή
        graphics::drawDisk(x, y + height / 2, base_offset, brush); // Βάση
        graphics::resetPose();

        // Σχεδίαση γραμμών που συνδέουν κορυφή και βάση
        graphics::drawLine(x - base_offset, y - height / 2, x - base_offset, y + height / 2, brush);
        graphics::drawLine(x + base_offset, y - height / 2, x + base_offset, y + height / 2, brush);
    }
    /**
     * Σχεδιάζει μία σφαίρα (αναπαρίσταται ως δίσκος και ελλείψεις).
     * @param x Η x-συντεταγμένη του κέντρου της σφαίρας.
     * @param y Η y-συντεταγμένη του κέντρου της σφαίρας.
     * @param radius Η ακτίνα της σφαίρας.
     * @param brush Το αντικείμενο Brush για το στυλ σχεδίασης.
     */
    void drawSphere(float x, float y, float radius, const graphics::Brush& brush) {
        graphics::drawDisk(x, y, radius, brush);
        graphics::setScale(1.0f, 0.5f);
        graphics::drawDisk(x, y, radius, brush);
        graphics::setOrientation(90);
        graphics::drawDisk(x, y, radius, brush);
        graphics::resetPose();
    }

    /**
   * Σχεδιάζει μία πυραμίδα σε προοπτική.
   * @param x Η x-συντεταγμένη του κέντρου της βάσης της πυραμίδας.
   * @param y Η y-συντεταγμένη του κέντρου της βάσης της πυραμίδας.
   * @param base_side Η πλευρά της τετράγωνης βάσης της πυραμίδας.
   * @param height Το ύψος της πυραμίδας.
   * @param brush Το αντικείμενο Brush για το στυλ σχεδίασης.
   */
    void drawPyramid(float x, float y, float base_side, float height, const graphics::Brush& brush) {
        // Ορισμός της κλίσης για καλύτερη προοπτική στην βάση της πυραμίδας
        float offsetX = base_side * 0.5f;  // μισό της πλευράς βάσης για συμμετρία
        float offsetY = base_side * 0.3f;  // μικρότερος συντελεστής για "συμπίεση" της προοπτικής

        // Σχεδίαση της βάσης της πυραμίδας ως παραλληλόγραμμο με προοπτική
        graphics::drawLine(x - offsetX, y - offsetY, x + offsetX, y - offsetY, brush); // πάνω πλευρά
        graphics::drawLine(x - offsetX, y + offsetY, x + offsetX, y + offsetY, brush); // κάτω πλευρά
        graphics::drawLine(x - offsetX, y - offsetY, x - offsetX, y + offsetY, brush); // αριστερή πλευρά
        graphics::drawLine(x + offsetX, y - offsetY, x + offsetX, y + offsetY, brush); // δεξιά πλευρά

        // Συντεταγμένες κορυφής της πυραμίδας
        float peakX = x;
        float peakY = y - height;

        // Σχεδίαση γραμμών από την κορυφή της πυραμίδας στις γωνίες της βάσης για καλύτερη προοπτική
        graphics::drawLine(peakX, peakY, x - offsetX, y - offsetY, brush); // από κορυφή σε αριστερή πάνω γωνία
        graphics::drawLine(peakX, peakY, x + offsetX, y - offsetY, brush); // από κορυφή σε δεξιά πάνω γωνία
        graphics::drawLine(peakX, peakY, x - offsetX, y + offsetY, brush); // από κορυφή σε αριστερή κάτω γωνία
        graphics::drawLine(peakX, peakY, x + offsetX, y + offsetY, brush); // από κορυφή σε δεξιά κάτω γωνία
    }


    /**
     * Σχεδιάζει ένα πρίσμα με κανονική βάση σε προοπτική.
     * @param x Η x-συντεταγμένη του κέντρου του πρίσματος.
     * @param y Η y-συντεταγμένη του κέντρου του πρίσματος.
     * @param sides Ο αριθμός των πλευρών της βάσης του πρίσματος.
     * @param radius Η ακτίνα της βάσης του πρίσματος.
     * @param height Το ύψος του πρίσματος.
     * @param brush Το αντικείμενο Brush για το στυλ σχεδίασης.
     */
    void drawPrism(float x, float y, int sides, float radius, float height, const graphics::Brush& brush) {
        std::vector<float> px(sides), py(sides);
        for (int i = 0; i < sides; i++) {
            float angle = 2 * M_PI * i / sides;
            px[i] = x + radius * cos(angle);
            py[i] = y + radius * sin(angle);
        }

        for (int i = 0; i < sides; i++) {
            graphics::drawLine(px[i], py[i], px[(i + 1) % sides], py[(i + 1) % sides], brush);
            graphics::drawLine(px[i], py[i], px[i], py[i] - height, brush);
        }

        for (int i = 0; i < sides; i++) {
            graphics::drawLine(px[i], py[i] - height, px[(i + 1) % sides], py[(i + 1) % sides] - height, brush);
        }
    }

} // namespace ThreeDShapes

#endif // THREE_D_SHAPES_H
